#!/bin/bash

# constants
T=TRUE
F=FALSE
N=NONE
PNGarg="-PNG"
PDFarg="exPDFbl"
PSDarg="exPSDbl"
# reqType values
PDF="PDF"
PSD="PSD"
# reqTypePSD values
RECS=R
TIMES=S
TOTAL=T

pgmName=`basename $0`
function usage() 
{
	which=$1
	case "$which" in
		"$PDFarg" )
			echo "Usage: "
			echo "  $which [HOST:]DBName NTW STN LOC CHN BL-Name Pct [ Start_DATE End_DATE ] [ "$PNGarg" ]"
			echo
			echo "  Example:	$which micros IU ANMO -- BHE HNM 30 2003-04-03 2004-05-03 "
			echo "	  Returns the PDF for the given channel having Pct % of PSD data points Outside the given Base-Line"
			echo "	  Output format: Period Power number_of_hits"
			echo
			echo "	'$PNGarg': Provide output in format intended as input to pqlxPNG plotting program"
			echo "	With this argument, the output can be piped thus:"
			echo "		$which ... '$PNGarg' | $PQLXBIN/pqlxPNG --inputPDF --pngName=name --pngDir=/tmp"
			echo "	See pqlxPNG for a complete list of options"
		;;
		"$PSDarg" )
			echo "Usage: "
			echo "  $which [HOST:]DBName NTW STN LOC CHN BL-Name Pct ( $RECS | $TIMES | $TOTAL ) [ Start_DATE End_DATE ]"
			echo
			echo "  Example:	$which micros IU ANMO -- BHE HNM 30 R 2003-04-03 2004-05-03"
			echo "    Returns PSD information for the given channel having Pct % of PSD data points Outside the given Base-Line"
			echo "      Output defined according to the argument:"
			echo "        R - Return all matching PSD records, Output format: DATE HOUR Period Power"
			echo "        S - Return PSD records, Output format: Period Power number_of_hits"
			echo "        T - Return Total Number of qualifying PSD records"
		;;
	esac
	echo
	exit
}

function PDFcheck()
{	# PDF request - specific check
	# confirm single sample rate across all PSD's
	# we make this check across the entire PSD, ignoring our period and power bounding arguments
	# first: count total periods for a single PSD, doesn't matter which
	if [ "$DATES" = "$T" ] ; then
		sql="SELECT p1.psd_pk from $PSDpsd p, $PSDidx p1 WHERE p1.psd_pk = p.psd_fk and "
		sql=$sql"p1.day >= '$SDate' and p1.day <= '$EDate' LIMIT 1"
	else
		sql="SELECT psd_pk from $PSDidx LIMIT 1"
	fi
	psdFK=`echo ${sql} | ${execCMD} | grep -v psd_pk`
	sql="SELECT count(period) cnt FROM $PSDpsd p where p.psd_fk = ${psdFK}"
	perCnt1=`echo ${sql} | ${execCMD} | grep -v cnt`
	# second: count the total period for all PSD's across requested date range
	if [ "$DATES" = "$T" ] ; then
		sql="SELECT count(distinct(p.period)) cnt from $PSDpsd p, $PSDidx p1 WHERE p1.psd_pk = p.psd_fk "
		sql=$sql"and p1.day >= '$SDate' and p1.day <= '$EDate'"
	else
		sql="SELECT count(distinct(p.period)) cnt from $PSDpsd p"
	fi
	perCntTtl=`echo ${sql} | ${execCMD} | grep -v cnt`
	
	if [ $perCnt1 -ne $perCntTtl ] ; then
	  echo "Period Count Mis-Match due to Existence of Multiple Sample Rates" >/dev/stderr
	  echo "To analyze this problem:" >/dev/stderr
	  echo "\tExecute script psdTableIntegrity.sh to clean PSD Tables, and/or" >/dev/stderr
	  echo "\tExecute script sampleRateTRC.sh to list the sample rate epochs" >/dev/stderr
	  echo >/dev/stderr
	  echo "For this channel, execution of said scripts would be:" >/dev/stderr
	  echo "\tbash> $PQLXBIN/psdTableIntegrity.sh $1 $NTWi $STNi $LOCi $CHNi" >/dev/stderr
	  echo "\tbash> $PQLXBIN/sampleRateTrc.sh $1 $NTWi $STNi $LOCi $CHNi $Sdate $Edate LIST" >/dev/stderr
	  echo >/dev/stderr
	  exit
	fi
	
	# PNG output valid for PDF request only
	if [ "$PNG" = "$T" ] ; then
		if [ "$DATES" = "$T" ] ; then
			sql="SELECT count(distinct(p1.psd_pk)) FROM $PSDpsd p, $PSDidx p1 WHERE p1.psd_pk = p.psd_fk and "
			sql=$sql"p1.day >= '$SDate' and p1.day <= '$EDate' and p.period >= $minFreq and p.period <= $maxFreq and "
			sqlNum=$sql"p.power >= $minPwr and p.power <= $maxPwr"
			sqlDates="SELECT YEAR('$SDate'), DAYOFYEAR('$SDate'), YEAR('$EDate'), DAYOFYEAR('$EDate')"
		else
			sqlNum="SELECT count(distinct(psd_pk)) from $PSDidx"
			sqlDates="SELECT YEAR(MIN(day)), DAYOFYEAR(MIN(day)), YEAR(MAX(day)), DAYOFYEAR(MAX(day)) FROM $PSDidx"
		fi
		numPSD=`echo ${sqlNum} | ${execCMD} | grep -v count`
		dates=`echo ${sqlDates} | ${execCMD} | grep -v YEAR`
		echo ${NTWi} ${STNi} ${LOCi} ${CHNi} $dates $numPSD
	fi
}

if [ $# -lt 7 ] ; then
	usage "$pgmName"
fi

case "$MYSQLBIN" in 
	"" )
		mysqlexec=mysql
	;;
	* )
		mysqlexec=$MYSQLBIN/mysql
	;;
esac

# defaults
PNG=$F
DATES=$F

# what's the type of request?
reqType=$N
reqTypePSD=$N
case "$pgmName" in
	"$PDFarg" )
		reqType=$PDF
	;;
	"$PSDarg" )
		reqType=$PSD
	;;
esac
if [ "$reqType" = "$N" ] ; then
	echo "Unknown extract type '$pgmName', Cannot continue"
	echo "Quitting without doing anything... Goodbye"
	exit
fi

# process command line
HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
shift
NTWi=$1 ; shift
STNi=$1 ; shift
LOCi=$1 ; shift
CHNi=$1 ; shift
BLn=$1 ; shift
PCT=$1 ; shift
case "$reqType" in
	"$PSD" )
		reqTypePSD=$1 ; shift
		case "$reqTypePSD" in
			"$RECS" | "$TIMES" | "$TOTAL" )
			;;
			* )		# unknown argument, quit
				usage $pgmName
			;;
		esac
	;;
esac

# any optional arguments?
case "$#" in
	"2" | "3" )
		SDate=$1 ; shift
		EDate=$1 ; shift
		DATES=$T
	;;
esac
case "$#" in
	"1" )
		if [ "$1" = "$PNGarg" ] ; then
			PNG=$T
		fi
	;;
esac

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

USERN=pqlxClnt
PWORD=PqLXClnT
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=$USERN --password=$PWORD --compress"

# confirm the channel is real
sqlStr="SELECT chni_pk FROM ${DBn}.nslc WHERE ntw = '$NTWi' and stn = '$STNi' and loc = '$LOCi' and chn = '$CHNi'"
chniFK=`echo $sqlStr | ${execCMD} | grep -v chni_pk`
if [ "$chniFK" = "" ] ; then
	echo "No channel instance found for $NTWi.$STNi.$LOCi.$CHNi, please try again..."
	echo "Quitting without doing anything... Goodbye"
	exit
fi

# confirm the baseline exists
sqlStr="SELECT bline_pk FROM ${DBn}.BLine WHERE name = '$BLn'"
blineFK=`echo $sqlStr | ${execCMD} | grep -v bline_pk`
if [ "$blineFK" = "" ] ; then
	echo "No Baseline by the name of '$BLn' found, please check and try again"
	echo "Quitting without doing anything... Goodbye"
	exit
fi

PSDidx=${DBn}DATA.psd${chniFK}idx
PSDpsd=${DBn}DATA.psd${chniFK}
BLStat=${DBn}STATS.BLineS${chniFK}

# PDF request - do a check before proceeding
if [ "$reqType" = "$PDF" ] ; then
	PDFcheck
fi

# make the SQL to execute
case "$reqType" in
	"$PDF" )
		sql="SELECT p.period, p.power, count(*) "
		if [ "$DATES" = "$T" ] ; then
			sql=$sql"FROM $PSDpsd p, $PSDidx p1, ${BLStat} b WHERE p1.psd_pk = p.psd_fk and "
			sql=$sql"p1.day >= '$SDate' and p1.day <= '$EDate' and "
		else
			sql=$sql"FROM $PSDpsd p, ${BLStat} b WHERE "
		fi
		sql=$sql"b.psd_fk = p.psd_fk and b.bline_fk = $blineFK and b.pct >= '$PCT' "
		sql=$sql"GROUP BY p.period, p.power ORDER BY p.period, p.power desc"
	;;
	"$PSD" )
		case "$reqTypePSD" in
			"$RECS" )
				sql="SELECT day, startT, period, power FROM $PSDpsd p, $PSDidx p1, ${BLStat} b "
				sql=$sql"WHERE p1.psd_pk = p.psd_fk and "
				if [ "$DATES" = "$T" ] ; then
					sql=$sql"p1.day >= '$SDate' and p1.day <= '$EDate' and "
				fi
				sql=$sql"b.psd_fk = p.psd_fk and b.bline_fk = $blineFK and b.pct >= '$PCT' "
				sql=$sql"GROUP BY p1.day, p1.startT, p.period "
				sql=$sql"ORDER BY p1.day, p1.startT, p.period"
			;;
			"$TIMES" )
				sql="SELECT day, startT, endT FROM $PSDidx p1, ${BLStat} b "
				sql=$sql"WHERE "
				if [ "$DATES" = "$T" ] ; then
					sql=$sql"p1.day >= '$SDate' and p1.day <= '$EDate' and "
				fi
				sql=$sql"b.psd_fk = p1.psd_pk and b.bline_fk = $blineFK and b.pct >= '$PCT' "
				sql=$sql"ORDER BY p1.day, p1.startT"
			;;
			"$TOTAL" )
				sql="SELECT count(*) cnt FROM $PSDidx p1, ${BLStat} b "
				sql=$sql"WHERE "
				if [ "$DATES" = "$T" ] ; then
					sql=$sql"p1.day >= '$SDate' and p1.day <= '$EDate' and "
				fi
				sql=$sql"b.psd_fk = p1.psd_pk and b.bline_fk = $blineFK and b.pct >= '$PCT' "
			;;
		esac
	;;
esac

echo $sql | ${execCMD} | grep -v period | grep -v day | grep -v cnt 