#!/bin/bash

if [ "$1" = "" ] || \
   [ "$2" != "ALL" -a "$2" != "ERRORS" ] || \
   [ "$2" = "ERRORS" -a "$3" != "SUMMARY" -a "$3" != "DETAIL" ] ; then
	echo "usage: $0 [HOST:]DBName ( ALL | ERRORS [SUMMARY | DETAIL])"
	echo
	echo "  Returns a list of either:"
	echo "    ALL: the current count of events on the EVENTQ table for each event"
	echo "    ERRORS: a list of all errors, per event, existing on the EVENTQ table:"
	echo "      output columns:	CHANNEL	EVENT	FILENAME	ERROR	(EVENTQ.evtq_id)"
	echo
	echo "example:	$0 ANSS ERRORS"
	echo "  Returns a list of the current errors in the EVENTQ for database ANSS"
	echo "(N.B. This script is non-invasive and can be executed at any time.)"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"

# execute the SQL command
case "$2" in
    ALL )
		tmpDir=/tmp/pqlxES
		if [ ! -e $tmpDir ] ; then
			mkdir $tmpDir
			chmod a+rwx $tmpDir
		fi
		if [ ! -e $tmpDir ] ; then
			echo "Unable to create directory $tmpDir, cannot continue..."
			exit
		fi
		tmpFile=$tmpDir/pqlxES.tmp
		/bin/rm -f $tmpFile &>/dev/null
		sql="select count(*) TOTAL, C1.description as 'EVENT', processed as 'STATUS', "
		sql=$sql"C2.description as 'ERROR' INTO OUTFILE '$tmpFile' FIELDS TERMINATED BY ',' "
		sql=$sql"LINES TERMINATED BY ',' FROM EVENTQ E, CONSTANT C1, CONSTANT C2 "
		sql=$sql"where E.error_no = C2.val and E.type = C1.val and C1.type = 'EVENT' and "
		sql=$sql"C2.type = 'ERRORMSG' group by 2, 3, 4 order by E.processed, E.type, E.error_no "
		echo $sql | ${execCMD}
		out="TOTAL,EVENT,STATUS,ERROR,"`cat $tmpFile`
		/bin/rm -f $tmpFile &>/dev/null
		colW=( 9 35 8 )
		numCols=${#colW[@]}
		index=0
		col=0
		while echo $out | grep \, &> /dev/null ; do
			str=${out%%\,*}
			let "len = ${colW[$col]} - ${#str}"
			case "$col" in
				0 )
					while [ "$len" -gt "0" ] ; do
						echo -n ' '
						let "len = $len - 1"
					done
					echo -n $str
					echo -n '  '
				;;
				[1-2] )
					echo -n $str
					while [ "$len" -gt "0" ] ; do
						echo -n ' '
						let "len = $len - 1"
					done
				;;
				3 )
					echo $str
					col=-1
				;;
			esac
			out=${out#*\,}
			let "col = $col + 1"
		done
		/bin/rm -f $tmpFile &>/dev/null
	;;

    ERRORS )
		case "$3" in
			DETAIL )
				outFile=$PQLXLOG/ED-$DBn-`date +%Y.%j.%H%M`.log
				msg="Detail output written to file $outFile"
				sql="SELECT CONCAT(n.ntw, '.', n.stn, '.', n.loc, '.', n.chn) as 'CHANNEL     ', C1.description as EVENT, "
				sql=$sql"T.filename as FILENAME, C2.description as ERROR, CONCAT('(',E.evtq_id,')') as EVTID "
				sql=$sql"FROM nslc n, TRC T, EVENTQ E, CONSTANT C1, CONSTANT C2 "
				sql=$sql"WHERE n.chni_pk = T.chni_fk AND T.trc_pk = E.typeKey and E.error_no=C2.val and "
				sql=$sql"E.error_no <> 0 and E.type < 10 and C2.type = 'ERRORMSG' and C1.type = 'EVENT' and C1.val = E.type "
				sql=$sql"UNION "
				sql=$sql"SELECT CONCAT(n.ntw, '.', n.stn, '.', n.loc, '.', n.chn) as 'CHANNEL     ', C1.description as EVENT, "
				sql=$sql"'NONE' as FILENAME, C2.description as ERROR, CONCAT('(',E.evtq_id,')') as EVTID "
				sql=$sql"FROM nslc n, EVENTQ E, CONSTANT C1, CONSTANT C2 "
				sql=$sql"WHERE n.chni_pk = E.typeKey and E.error_no=C2.val and E.error_no <> 0 and "
				sql=$sql"E.type > 10 and C2.type = 'ERRORMSG' and C1.type = 'EVENT' and C1.val = E.type order by 1, 2;"
				echo $sql | ${execCMD} | grep -v EVTID >$outFile
			;;
			SUMMARY )
				outFile=`pqlx-ErrorSummary.sh $1`
				msg="Summary output written to file $outFile"
			;;
		esac
		
		echo $msg
	;;
esac

